/*!
 * Jasny Bootstrap Extensions j3
 *
 * Copyright 2012 Jasny BV
 * Licensed under the Apache License v2.0
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Extended with pride by @ArnoldDaniels of jasny.net
 */

.clearfix {
  *zoom: 1;
}

.clearfix:before,
.clearfix:after {
  display: table;
  line-height: 0;
  content: "";
}

.clearfix:after {
  clear: both;
}

.hide-text {
  font: 0/0 a;
  color: transparent;
  text-shadow: none;
  background-color: transparent;
  border: 0;
}

.input-block-level {
  display: block;
  width: 100%;
  min-height: 30px;
  -webkit-box-sizing: border-box;
     -moz-box-sizing: border-box;
          box-sizing: border-box;
}

.container-semifluid {
  max-width: 940px;
  padding-right: 20px;
  padding-left: 20px;
  margin-right: auto;
  margin-left: auto;
  *zoom: 1;
}

.container-semifluid:before,
.container-semifluid:after {
  display: table;
  line-height: 0;
  content: "";
}

.container-semifluid:after {
  clear: both;
}

form > *:last-child {
  margin-bottom: 0;
}

label input[type="image"],
label input[type="checkbox"],
label input[type="radio"] {
  vertical-align: middle;
}

select {
  padding-left: 2px;
}

.small-labels .control-group > label {
  width: 70px;
}

.small-labels .controls {
  margin-left: 80px;
}

.small-labels .form-actions {
  padding-left: 80px;
}

.form-vertical .form-horizontal .control-group > label {
  text-align: left;
}

.form-horizontal .form-vertical .control-group > label {
  float: none;
  padding-top: 0;
  text-align: left;
}

.form-horizontal .form-vertical .controls {
  margin-left: 0;
}

.form-horizontal .form-vertical.form-actions,
.form-horizontal .form-vertical .form-actions {
  padding-left: 20px;
}

.control-group .control-group {
  margin-bottom: 0;
}

.form-horizontal .well .control-label {
  width: 120px;
}

.form-horizontal .well .controls {
  margin-left: 140px;
}

form .well > *:last-child {
  margin-bottom: 0;
}

.editor {
  width: 100%;
  height: 100px;
  padding: 5px;
  -webkit-box-sizing: border-box;
     -moz-box-sizing: border-box;
          box-sizing: border-box;
}

.uneditable-textarea.editor-html {
  padding: 5px 3px 5px 5px;
  white-space: normal;
}

textarea.editor-html {
  visibility: hidden;
}

.uneditable-input,
.uneditable-textarea {
  display: inline-block;
  padding: 4px 3px 4px 5px;
  font-size: 14px;
  line-height: 20px;
  color: #555555;
  cursor: not-allowed;
  background-color: #ffffff;
  border: 1px solid #eee;
  -webkit-border-radius: 4px;
     -moz-border-radius: 4px;
          border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
     -moz-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
     -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
          box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
}

.uneditable-input {
  height: 20px;
  overflow: hidden;
  white-space: pre;
}

.uneditable-textarea {
  overflow-x: hidden;
  overflow-y: auto;
  white-space: pre-wrap;
}

select[disabled],
textarea[disabled],
input[type="text"][disabled],
input[type="password"][disabled],
input[type="datetime"][disabled],
input[type="datetime-local"][disabled],
input[type="date"][disabled],
input[type="month"][disabled],
input[type="time"][disabled],
input[type="week"][disabled],
input[type="number"][disabled],
input[type="email"][disabled],
input[type="url"][disabled],
input[type="search"][disabled] {
  color: #999;
}

.uneditable-input.disabled,
.uneditable-textarea.disabled {
  color: #999;
  cursor: not-allowed;
  background-color: #f5f5f5;
  border-color: #ddd;
}

textarea,
.uneditable-textarea {
  height: 60px;
}

textarea[rows="1"],
.uneditable-textarea[rows="1"] {
  height: 40px;
}

textarea[rows="2"],
.uneditable-textarea[rows="2"] {
  height: 60px;
}

textarea[rows="3"],
.uneditable-textarea[rows="3"] {
  height: 80px;
}

textarea[rows="4"],
.uneditable-textarea[rows="4"] {
  height: 100px;
}

textarea[rows="5"],
.uneditable-textarea[rows="5"] {
  height: 120px;
}

textarea[rows="6"],
.uneditable-textarea[rows="6"] {
  height: 140px;
}

textarea[rows="7"],
.uneditable-textarea[rows="7"] {
  height: 160px;
}

textarea[rows="8"],
.uneditable-textarea[rows="8"] {
  height: 180px;
}

textarea[rows="9"],
.uneditable-textarea[rows="9"] {
  height: 200px;
}

textarea[rows="10"],
.uneditable-textarea[rows="10"] {
  height: 220px;
}

textarea[rows="11"],
.uneditable-textarea[rows="11"] {
  height: 240px;
}

textarea[rows="12"],
.uneditable-textarea[rows="12"] {
  height: 260px;
}

textarea[rows="13"],
.uneditable-textarea[rows="13"] {
  height: 280px;
}

textarea[rows="14"],
.uneditable-textarea[rows="14"] {
  height: 300px;
}

textarea[rows="15"],
.uneditable-textarea[rows="15"] {
  height: 320px;
}

textarea[rows="16"],
.uneditable-textarea[rows="16"] {
  height: 340px;
}

textarea[rows="17"],
.uneditable-textarea[rows="17"] {
  height: 360px;
}

textarea[rows="18"],
.uneditable-textarea[rows="18"] {
  height: 380px;
}

textarea[rows="19"],
.uneditable-textarea[rows="19"] {
  height: 400px;
}

textarea[rows="20"],
.uneditable-textarea[rows="20"] {
  height: 420px;
}

textarea[rows="21"],
.uneditable-textarea[rows="21"] {
  height: 440px;
}

textarea[rows="22"],
.uneditable-textarea[rows="22"] {
  height: 460px;
}

textarea[rows="23"],
.uneditable-textarea[rows="23"] {
  height: 480px;
}

textarea[rows="24"],
.uneditable-textarea[rows="24"] {
  height: 500px;
}

textarea[rows="25"],
.uneditable-textarea[rows="25"] {
  height: 520px;
}

textarea[rows="26"],
.uneditable-textarea[rows="26"] {
  height: 540px;
}

textarea[rows="27"],
.uneditable-textarea[rows="27"] {
  height: 560px;
}

textarea[rows="28"],
.uneditable-textarea[rows="28"] {
  height: 580px;
}

textarea[rows="29"],
.uneditable-textarea[rows="29"] {
  height: 600px;
}

textarea[rows="30"],
.uneditable-textarea[rows="30"] {
  height: 620px;
}

textarea[rows="35"],
.uneditable-textarea[rows="35"] {
  height: 720px;
}

textarea[rows="40"],
.uneditable-textarea[rows="40"] {
  height: 820px;
}

textarea[rows="45"],
.uneditable-textarea[rows="45"] {
  height: 920px;
}

textarea[rows="50"],
.uneditable-textarea[rows="50"] {
  height: 1020px;
}

textarea[rows="55"],
.uneditable-textarea[rows="55"] {
  height: 1120px;
}

textarea[rows="60"],
.uneditable-textarea[rows="60"] {
  height: 1220px;
}

textarea[rows="65"],
.uneditable-textarea[rows="65"] {
  height: 1320px;
}

textarea[rows="70"],
.uneditable-textarea[rows="70"] {
  height: 1420px;
}

textarea[rows="75"],
.uneditable-textarea[rows="75"] {
  height: 1520px;
}

textarea[rows="80"],
.uneditable-textarea[rows="80"] {
  height: 1620px;
}

textarea[rows="85"],
.uneditable-textarea[rows="85"] {
  height: 1720px;
}

textarea[rows="90"],
.uneditable-textarea[rows="90"] {
  height: 1820px;
}

textarea[rows="95"],
.uneditable-textarea[rows="95"] {
  height: 1920px;
}

textarea[rows="100"],
.uneditable-textarea[rows="100"] {
  height: 2020px;
}

.uneditable-textarea {
  -webkit-box-sizing: border-box;
     -moz-box-sizing: border-box;
          box-sizing: border-box;
}

.uneditable-input[class*="span"],
.uneditable-textarea[class*="span"],
.row-fluid .uneditable-input[class*="span"],
.row-fluid .uneditable-textarea[class*="span"] {
  float: none;
  margin-left: 0;
}

.input-append .uneditable-input,
.input-prepend .uneditable-input {
  vertical-align: top;
}

.input-append .uneditable-input[class*="span"],
.input-prepend .uneditable-input[class*="span"] {
  display: inline-block;
}

.uneditable-form input[disabled],
.uneditable-form textarea[disabled],
.uneditable-form select[disabled] {
  cursor: auto;
}

.uneditable-form .uneditable-input,
.uneditable-form .uneditable-textarea {
  cursor: text;
}

.uneditable-form .form-actions {
  background-color: transparent;
}

.header-actions {
  padding: 0 20px;
  line-height: 36px;
}

.table-actions {
  padding-bottom: 20px;
  *zoom: 1;
}

.table-actions:before,
.table-actions:after {
  display: table;
  line-height: 0;
  content: "";
}

.table-actions:after {
  clear: both;
}

tr.rowlink td {
  cursor: pointer;
}

tr.rowlink td.nolink {
  cursor: auto;
}

.table tbody tr.rowlink:hover td {
  background-color: #cfcfcf;
}

a.rowlink {
  font: inherit;
  color: inherit;
  text-decoration: inherit;
}

.act {
  display: inline;
  padding: 0;
  font-weight: bold;
  color: #555555;
  background: inherit;
  border: none;
  -webkit-transition: text-shadow 0.1s linear;
     -moz-transition: text-shadow 0.1s linear;
       -o-transition: text-shadow 0.1s linear;
          transition: text-shadow 0.1s linear;
}

.act:hover {
  color: #333333;
  text-decoration: none;
  text-shadow: 1px 1px 3px rgba(85, 85, 85, 0.5);
}

.act-primary {
  color: #006dcc;
}

.act-primary:hover {
  color: #0044cc;
  text-shadow: 1px 1px 3px rgba(0, 109, 204, 0.5);
}

.act-info {
  color: #49afcd;
}

.act-info:hover {
  color: #2f96b4;
  text-shadow: 1px 1px 3px rgba(75, 175, 206, 0.5);
}

.act-success {
  color: #51a351;
}

.act-success:hover {
  color: #468847;
  text-shadow: 1px 1px 3px rgba(81, 164, 81, 0.5);
}

.act-warning {
  color: #c09853;
}

.act-warning:hover {
  color: #f89406;
  text-shadow: 1px 1px 3px rgba(192, 152, 84, 0.5);
}

.act-danger {
  color: #b94a48;
}

.act-danger:hover {
  color: #bd362f;
  text-shadow: 1px 1px 3px rgba(185, 72, 70, 0.5);
}

.act.disabled,
.act[disabled] {
  color: #AAAAAA;
  cursor: not-allowed;
}

.act.disabled:hover,
.act[disabled]:hover {
  color: #AAAAAA;
  text-shadow: none;
}

.form-actions .act {
  line-height: 30px;
}

@font-face {
  font-family: IconicStroke;
  font-weight: normal;
  src: url(../font/iconic_stroke.eot);
  src: local('IconicStroke'), url(../font/iconic_stroke.eot?#iefix) format('embedded-opentype'), url(../font/iconic_stroke.woff) format('woff'), url(../font/iconic_stroke.ttf) format('truetype'), url(../font/iconic_stroke.svg#iconic) format('svg'), url(../font/iconic_stroke.otf) format('opentype');
}

@font-face {
  font-family: IconicFill;
  font-weight: normal;
  src: url(../font/iconic_fill.eot);
  src: local('IconicFill'), url(../font/iconic_fill.eot?#iefix) format('embedded-opentype'), url(../font/iconic_fill.woff) format('woff'), url(../font/iconic_fill.ttf) format('truetype'), url(../font/iconic_fill.svg#iconic) format('svg'), url(../font/iconic_fill.otf) format('opentype');
}

@media screen, print {
  [class*="iconic-"] {
    font-style: inherit;
    font-weight: normal;
    vertical-align: bottom;
  }
  [class*="iconic-"]:before {
    display: inline-block;
    width: 1em;
    font-family: IconicFill;
    font-size: 0.9em;
    text-align: center;
    vertical-align: middle;
    content: "";
  }
  .iconic-stroke:before {
    font-family: IconicStroke;
  }
  .iconic-hash:before {
    content: '\23';
  }
  .iconic-question-mark:before {
    content: '\3f';
  }
  .iconic-at:before {
    content: '\40';
  }
  .iconic-pilcrow:before {
    content: '\b6';
  }
  .iconic-info:before {
    content: '\2139';
  }
  .iconic-arrow-left:before {
    content: '\2190';
  }
  .iconic-arrow-up:before {
    content: '\2191';
  }
  .iconic-arrow-right:before {
    content: '\2192';
  }
  .iconic-arrow-down:before {
    content: '\2193';
  }
  .iconic-home:before {
    content: '\2302';
  }
  .iconic-sun:before {
    content: '\2600';
  }
  .iconic-cloud:before {
    content: '\2601';
  }
  .iconic-umbrella:before {
    content: '\2602';
  }
  .iconic-star:before {
    content: '\2605';
  }
  .iconic-moon:before {
    content: '\263e';
  }
  .iconic-heart:before {
    content: '\2764';
  }
  .iconic-cog:before {
    content: '\2699';
  }
  .iconic-bolt:before {
    content: '\26a1';
  }
  .iconic-key:before {
    content: '\26bf';
  }
  .iconic-rain:before {
    content: '\26c6';
  }
  .iconic-denied:before {
    content: '\26d4';
  }
  .iconic-mail:before {
    content: '\2709';
  }
  .iconic-pen:before {
    content: '\270e';
  }
  .iconic-x:before {
    content: '\2717';
  }
  .iconic-o-x:before {
    content: '\2718';
  }
  .iconic-check:before {
    content: '\2713';
  }
  .iconic-o-check:before {
    content: '\2714';
  }
  .iconic-left-quote:before {
    content: '\275d';
  }
  .iconic-right-quote:before {
    content: '\275e';
  }
  .iconic-plus:before {
    content: '\2795';
  }
  .iconic-minus:before {
    content: '\2796';
  }
  .iconic-curved-arrow:before {
    content: '\2935';
  }
  .iconic-document-alt:before {
    content: '\e000';
  }
  .iconic-calendar:before {
    content: '\e001';
  }
  .iconic-map-pin-alt:before {
    content: '\e002';
  }
  .iconic-comment-alt1:before {
    content: '\e003';
  }
  .iconic-comment-alt2:before {
    content: '\e004';
  }
  .iconic-pen-alt:before {
    content: '\e005';
  }
  .iconic-pen-alt2:before {
    content: '\e006';
  }
  .iconic-chat-alt:before {
    content: '\e007';
  }
  .iconic-o-plus:before {
    content: '\e008';
  }
  .iconic-o-minus:before {
    content: '\e009';
  }
  .iconic-bars-alt:before {
    content: '\e00a';
  }
  .iconic-book-alt:before {
    content: '\e00b';
  }
  .iconic-aperture-alt:before {
    content: '\e00c';
  }
  .iconic-beaker-alt:before {
    content: '\e010';
  }
  .iconic-left-quote-alt:before {
    content: '\e011';
  }
  .iconic-right-quote-alt:before {
    content: '\e012';
  }
  .iconic-o-arrow-left:before {
    content: '\e013';
  }
  .iconic-o-arrow-up:before {
    content: '\e014';
  }
  .iconic-o-arrow-right:before {
    content: '\e015';
  }
  .iconic-o-arrow-down:before {
    content: '\e016';
  }
  .iconic-o-arrow-left-alt:before {
    content: '\e017';
  }
  .iconic-o-arrow-up-alt:before {
    content: '\e018';
  }
  .iconic-o-arrow-right-alt:before {
    content: '\e019';
  }
  .iconic-o-arrow-down-alt:before {
    content: '\e01a';
  }
  .iconic-brush:before {
    content: '\e01b';
  }
  .iconic-brush-alt:before {
    content: '\e01c';
  }
  .iconic-eyedropper:before {
    content: '\e01e';
  }
  .iconic-layers:before {
    content: '\e01f';
  }
  .iconic-layers-alt:before {
    content: '\e020';
  }
  .iconic-compass:before {
    content: '\e021';
  }
  .iconic-award:before {
    content: '\e022';
  }
  .iconic-beaker:before {
    content: '\e023';
  }
  .iconic-steering-wheel:before {
    content: '\e024';
  }
  .iconic-eye:before {
    content: '\e025';
  }
  .iconic-aperture:before {
    content: '\e026';
  }
  .iconic-image:before {
    content: '\e027';
  }
  .iconic-chart:before {
    content: '\e028';
  }
  .iconic-chart-alt:before {
    content: '\e029';
  }
  .iconic-target:before {
    content: '\e02a';
  }
  .iconic-tag:before {
    content: '\e02b';
  }
  .iconic-rss:before {
    content: '\e02c';
  }
  .iconic-rss-alt:before {
    content: '\e02d';
  }
  .iconic-share:before {
    content: '\e02e';
  }
  .iconic-undo:before {
    content: '\e02f';
  }
  .iconic-reload:before {
    content: '\e030';
  }
  .iconic-reload-alt:before {
    content: '\e031';
  }
  .iconic-loop:before {
    content: '\e032';
  }
  .iconic-loop-alt:before {
    content: '\e033';
  }
  .iconic-back-forth:before {
    content: '\e034';
  }
  .iconic-back-forth-alt:before {
    content: '\e035';
  }
  .iconic-spin:before {
    content: '\e036';
  }
  .iconic-spin-alt:before {
    content: '\e037';
  }
  .iconic-move-horizontal:before {
    content: '\e038';
  }
  .iconic-move-horizontal-alt:before {
    content: '\e039';
  }
  .iconic-o-move-horizontal:before {
    content: '\e03a';
  }
  .iconic-move-vertical:before {
    content: '\e03b';
  }
  .iconic-move-vertical-alt:before {
    content: '\e03c';
  }
  .iconic-o-move-vertical:before {
    content: '\e03d';
  }
  .iconic-move:before {
    content: '\e03e';
  }
  .iconic-move-alt:before {
    content: '\e03f';
  }
  .iconic-o-move:before {
    content: '\e040';
  }
  .iconic-transfer:before {
    content: '\e041';
  }
  .iconic-download:before {
    content: '\e042';
  }
  .iconic-upload:before {
    content: '\e043';
  }
  .iconic-cloud-download:before {
    content: '\e044';
  }
  .iconic-cloud-upload:before {
    content: '\e045';
  }
  .iconic-fork:before {
    content: '\e046';
  }
  .iconic-play:before {
    content: '\e047';
  }
  .iconic-o-play:before {
    content: '\e048';
  }
  .iconic-pause:before {
    content: '\e049';
  }
  .iconic-stop:before {
    content: '\e04a';
  }
  .iconic-eject:before {
    content: '\e04b';
  }
  .iconic-first:before {
    content: '\e04c';
  }
  .iconic-last:before {
    content: '\e04d';
  }
  .iconic-fullscreen:before {
    content: '\e04e';
  }
  .iconic-fullscreen-alt:before {
    content: '\e04f';
  }
  .iconic-fullscreen-exit:before {
    content: '\e050';
  }
  .iconic-fullscreen-exit-alt:before {
    content: '\e051';
  }
  .iconic-equalizer:before {
    content: '\e052';
  }
  .iconic-article:before {
    content: '\e053';
  }
  .iconic-read-more:before {
    content: '\e054';
  }
  .iconic-list:before {
    content: '\e055';
  }
  .iconic-list-nested:before {
    content: '\e056';
  }
  .iconic-cursor:before {
    content: '\e057';
  }
  .iconic-dial:before {
    content: '\e058';
  }
  .iconic-new-window:before {
    content: '\e059';
  }
  .iconic-trash:before {
    content: '\e05a';
  }
  .iconic-battery-half:before {
    content: '\e05b';
  }
  .iconic-battery-empty:before {
    content: '\e05c';
  }
  .iconic-battery-charging:before {
    content: '\e05d';
  }
  .iconic-chat:before {
    content: '\e05e';
  }
  .iconic-mic:before {
    content: '\e05f';
  }
  .iconic-movie:before {
    content: '\e060';
  }
  .iconic-headphones:before {
    content: '\e061';
  }
  .iconic-user:before {
    content: '\e062';
  }
  .iconic-lightbulb:before {
    content: '\e063';
  }
  .iconic-cd:before {
    content: '\e064';
  }
  .iconic-folder:before {
    content: '\e065';
  }
  .iconic-document:before {
    content: '\e066';
  }
  .iconic-pin:before {
    content: '\e067';
  }
  .iconic-map-pin:before {
    content: '\e068';
  }
  .iconic-book:before {
    content: '\e069';
  }
  .iconic-book-alt2:before {
    content: '\e06a';
  }
  .iconic-box:before {
    content: '\e06b';
  }
  .iconic-calendar-alt:before {
    content: '\e06c';
  }
  .iconic-comment:before {
    content: '\e06d';
  }
  .iconic-iphone:before {
    content: '\e06e';
  }
  .iconic-bars:before {
    content: '\e06f';
  }
  .iconic-camera:before {
    content: '\e070';
  }
  .iconic-volume-mute:before {
    content: '\e071';
  }
  .iconic-volume:before {
    content: '\e072';
  }
  .iconic-battery-full:before {
    content: '\e073';
  }
  .iconic-magnifying-glass:before {
    content: '\e074';
  }
  .iconic-lock:before {
    content: '\e075';
  }
  .iconic-unlock:before {
    content: '\e076';
  }
  .iconic-link:before {
    content: '\e077';
  }
  .iconic-wrench:before {
    content: '\e078';
  }
  .iconic-clock:before {
    content: '\e079';
  }
  .iconic-sun-stroke:before {
    font-family: IconicStroke;
    content: '\2600';
  }
  .iconic-moon-stroke:before {
    font-family: IconicStroke;
    content: '\263e';
  }
  .iconic-star-stroke:before {
    font-family: IconicStroke;
    content: '\2605';
  }
  .iconic-heart-stroke:before {
    font-family: IconicStroke;
    content: '\2764';
  }
  .iconic-key-stroke:before {
    font-family: IconicStroke;
    content: '\26bf';
  }
  .iconic-document-alt-stroke:before {
    font-family: IconicStroke;
    content: '\e000';
  }
  .iconic-comment-alt1-stroke:before {
    font-family: IconicStroke;
    content: '\e003';
  }
  .iconic-comment-alt2-stroke:before {
    font-family: IconicStroke;
    content: '\e004';
  }
  .iconic-pen-alt-stroke:before {
    font-family: IconicStroke;
    content: '\e005';
  }
  .iconic-chat-alt-stroke:before {
    font-family: IconicStroke;
    content: '\e007';
  }
  .iconic-award-stroke:before {
    font-family: IconicStroke;
    content: '\e022';
  }
  .iconic-tag-stroke:before {
    font-family: IconicStroke;
    content: '\e02b';
  }
  .iconic-trash-stroke:before {
    font-family: IconicStroke;
    content: '\e05a';
  }
  .iconic-folder-stroke:before {
    font-family: IconicStroke;
    content: '\e065';
  }
  .iconic-document-stroke:before {
    font-family: IconicStroke;
    content: '\e066';
  }
  .iconic-map-pin-stroke:before {
    font-family: IconicStroke;
    content: '\e068';
  }
  .iconic-calendar-alt-stroke:before {
    font-family: IconicStroke;
    content: '\e06c';
  }
  .iconic-comment-stroke:before {
    font-family: IconicStroke;
    content: '\e06d';
  }
  .iconic-lock-stroke:before {
    font-family: IconicStroke;
    content: '\e075';
  }
  .iconic-unlock-stroke:before {
    font-family: IconicStroke;
    content: '\e076';
  }
}

.page-alert {
  position: absolute;
  top: 0;
  left: 50%;
  z-index: 1020;
  width: 0;
}

.page-alert .alert {
  width: 550px;
  margin-left: -300px;
  border-top-width: 0;
  -webkit-border-radius: 0 0 4px 4px;
     -moz-border-radius: 0 0 4px 4px;
          border-radius: 0 0 4px 4px;
}

.navbar-fixed-top + .page-alert {
  top: 40px;
}

body > .page-alert {
  position: fixed;
}

.btn-file {
  position: relative;
  overflow: hidden;
  vertical-align: middle;
}

.btn-file > input {
  position: absolute;
  top: 0;
  right: 0;
  margin: 0;
  font-size: 23px;
  cursor: pointer;
  opacity: 0;
  filter: alpha(opacity=0);
  transform: translate(-300px, 0) scale(4);
  direction: ltr;
}

.fileupload {
  margin-bottom: 9px;
}

.fileupload .uneditable-input {
  display: inline-block;
  margin-bottom: 0;
  vertical-align: middle;
  cursor: text;
}

.fileupload .thumbnail {
  display: inline-block;
  margin-bottom: 5px;
  overflow: hidden;
  text-align: center;
  vertical-align: middle;
}

.fileupload .thumbnail > img {
  display: inline-block;
  max-height: 100%;
  vertical-align: middle;
}

.fileupload .btn {
  vertical-align: middle;
}

.fileupload-exists .fileupload-new,
.fileupload-new .fileupload-exists {
  display: none;
}

.fileupload-inline .fileupload-controls {
  display: inline;
}

.fileupload-new .input-append .btn-file {
  -webkit-border-radius: 0 3px 3px 0;
     -moz-border-radius: 0 3px 3px 0;
          border-radius: 0 3px 3px 0;
}

.thumbnail-borderless .thumbnail {
  padding: 0;
  border: none;
  -webkit-border-radius: 0;
     -moz-border-radius: 0;
          border-radius: 0;
  -webkit-box-shadow: none;
     -moz-box-shadow: none;
          box-shadow: none;
}

.fileupload-new.thumbnail-borderless .thumbnail {
  border: 1px solid #ddd;
}

.control-group.warning .fileupload .uneditable-input {
  color: #a47e3c;
  border-color: #a47e3c;
}

.control-group.warning .fileupload .fileupload-preview {
  color: #a47e3c;
}

.control-group.warning .fileupload .thumbnail {
  border-color: #a47e3c;
}

.control-group.error .fileupload .uneditable-input {
  color: #b94a48;
  border-color: #b94a48;
}

.control-group.error .fileupload .fileupload-preview {
  color: #b94a48;
}

.control-group.error .fileupload .thumbnail {
  border-color: #b94a48;
}

.control-group.success .fileupload .uneditable-input {
  color: #468847;
  border-color: #468847;
}

.control-group.success .fileupload .fileupload-preview {
  color: #468847;
}

.control-group.success .fileupload .thumbnail {
  border-color: #468847;
}

.nav-tabs > li > a,
.nav-pills > li > a {
  outline: none;
}

.nav-tabs > li.disabled > a {
  color: #CCCCCC;
  cursor: not-allowed;
}

.tabbable {
  border-color: #ddd;
  border-style: solid;
  border-width: 0;
  *zoom: 1;
}

.tabbable:before,
.tabbable:after {
  display: table;
  line-height: 0;
  content: "";
}

.tabbable:after {
  clear: both;
}

.tabbable > .nav-tabs {
  margin: 0;
}

.tab-content {
  padding: 18px 0 0 0;
  overflow: auto;
  border-color: #ddd;
  border-style: solid;
  border-width: 0;
}

.tabbable-bordered {
  -webkit-border-radius: 4px;
     -moz-border-radius: 4px;
          border-radius: 4px;
}

.tabbable-bordered > .tab-content {
  padding: 20px 20px 10px 20px;
  border-width: 0 1px 1px 1px;
  -webkit-border-radius: 0 0 4px 4px;
     -moz-border-radius: 0 0 4px 4px;
          border-radius: 0 0 4px 4px;
}

body > .container.tabbable > .nav-tabs {
  padding-top: 15px;
}

.tabs-below > .tab-content {
  padding: 0 0 10px 0;
}

.tabs-below.tabbable-bordered > .tab-content {
  padding: 20px 20px 10px 20px;
  border-width: 1px 1px 0 1px;
  -webkit-border-radius: 4px 4px 0 0;
     -moz-border-radius: 4px 4px 0 0;
          border-radius: 4px 4px 0 0;
}

body > .container.tabs-below.tabbable-bodered > .tab-content {
  border-top-width: 0;
  -webkit-border-radius: 0;
     -moz-border-radius: 0;
          border-radius: 0;
}

.tabs-left,
.tabs-right {
  margin-bottom: 20px;
}

.tabs-left > .nav-tabs,
.tabs-right > .nav-tabs {
  position: relative;
  z-index: 1;
  margin-bottom: 0;
}

.tabs-left > .tab-content,
.tabs-right > .tab-content {
  overflow: hidden;
}

.tabs-left > .nav-tabs {
  left: 1px;
}

.tabs-left > .nav-tabs > .active > a,
.tabs-left > .nav-tabs > .active > a:hover {
  border-color: #ddd transparent #ddd #ddd;
  *border-right-color: #ffffff;
}

.tabs-left > .tab-content {
  padding: 0 0 0 19px;
  border-left-width: 1px;
}

.tabs-left.tabbable-bordered {
  border-width: 0 1px 0 0;
}

.tabs-left.tabbable-bordered > .tab-content {
  padding: 20px 20px 10px 20px;
  border-width: 1px 0 1px 1px;
  -webkit-border-radius: 0 4px 4px 0;
     -moz-border-radius: 0 4px 4px 0;
          border-radius: 0 4px 4px 0;
}

body > .container.tabs-left.tabbable-bodered > .tab-content {
  border-top-width: 0;
  -webkit-border-radius: 0 0 4px 0;
     -moz-border-radius: 0 0 4px 0;
          border-radius: 0 0 4px 0;
}

.tabs-right > .nav-tabs {
  right: 1px;
}

.tabs-right > .nav-tabs > .active > a,
.tabs-right > .nav-tabs > .active > a:hover {
  border-color: #ddd #ddd #ddd transparent;
  *border-left-color: #ffffff;
}

.tabs-right > .tab-content {
  padding: 0 19px 0 0;
  border-right-width: 1px;
}

.tabs-right.tabbable-bordered {
  border-width: 0 0 0 1px;
}

.tabs-right.tabbable-bordered > .tab-content {
  padding: 20px 20px 10px 20px;
  border-width: 1px 1px 1px 0;
  -webkit-border-radius: 4px 0 0 4px;
     -moz-border-radius: 4px 0 0 4px;
          border-radius: 4px 0 0 4px;
}

body > .container.tabs-right.tabbable-bodered > .tab-content {
  border-top-width: 0;
  -webkit-border-radius: 0 0 0 4px;
     -moz-border-radius: 0 0 0 4px;
          border-radius: 0 0 0 4px;
}
