{% extends "base.html" %}

{% block content %}
<h2>Login</h2>
{% if error %}<p>{{ error }}</p>{% endif %}
<form method="POST" action="{{url_for('login')}}">
    <div class="input-prepend">
        <span class="add-on"><i class="fa fa-key"></i></span>
        <input class="span2" name="password" type="password" placeholder="Password">
        <button class="btn" type="submit">Login</button>
    </div>
</form>
{% endblock %}