body,div,h1,h2,h3,h4,h5,h6,p,blockquote,pre,dl,dt,dd,ol,ul,li,fieldset,form,label,legend,th,td,article,aside,figure,footer,header,hgroup,menu,nav,section{margin:0;padding:0;border:0}article,aside,details,figcaption,figure,footer,header,hgroup,nav,section,audio,canvas,video{display:block}html{height:100%;-webkit-text-size-adjust:100%;-ms-text-size-adjust:100%}body{min-height:100%;font-size:100%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sup{top:-0.5em}sub{bottom:-0.25em}pre{white-space:pre;white-space:pre-wrap;word-wrap:break-word}b,strong{font-weight:bold}abbr[title]{border-bottom:1px dotted}table{border-collapse:collapse;border-spacing:0}a img,img{-ms-interpolation-mode:bicubic;border:0}input,textarea,button,select{margin:0;font-size:100%;line-height:normal;vertical-align:baseline}button,html input[type="button"],input[type="reset"],input[type="submit"]{cursor:pointer;-webkit-appearance:button}input[type="checkbox"],input[type="radio"]{-webkit-box-sizing:border-box;-moz-box-sizing:border-box;-o-box-sizing:border-box;-ms-box-sizing:border-box;box-sizing:border-box}textarea{overflow:auto}::selection{background:red;color:white}::-moz-selection{background:red;color:white}img::selection{background:transparent}img::-moz-selection{background:transparent}body{-webkit-tap-highlight-color:rgba(255,0,0,0.62)}@media screen and (max-width:1px){body{font-size:.875em}}@media screen and (max-width:1px){body{font-size:1.125em}}.w1{width:3.75em}.w2{width:8.75em}.w3{width:13.75em}.w4{width:18.75em}.w5{width:23.75em}.w6{width:28.75em}.w7{width:33.75em}.w8{width:38.75em}.w9{width:43.75em}.w10{width:48.75em}.w11{width:53.75em}.w12{width:58.75em}.h1{height:3.75em}.h2{height:8.75em}.h3{height:13.75em}.h4{height:18.75em}.h5{height:23.75em}.h6{height:28.75em}.h7{height:33.75em}.h8{height:38.75em}.h9{height:43.75em}.h10{height:48.75em}.h11{height:53.75em}.h12{height:58.75em}.col0{left:0}.col1{left:3.75em}.col2{left:8.75em}.col3{left:13.75em}.col4{left:18.75em}.col5{left:23.75em}.col6{left:28.75em}.col7{left:33.75em}.col8{left:38.75em}.col9{left:43.75em}.col10{left:48.75em}.col11{left:53.75em}.col12{left:58.75em}.row0{top:0}.row1{top:3.75em}.row2{top:8.75em}.row3{top:13.75em}.row4{top:18.75em}.row5{top:23.75em}.row6{top:28.75em}.row7{top:33.75em}.row8{top:38.75em}.row9{top:43.75em}