{% extends "base.html" %}
{% import "macros/create_star.html" as create_macro %}

<!-- TODO: get header from page as well-->

{% block content %}
<section class="col0 row1 w3 h1">
<h2><i class="fa fa-magic"></i> Create Star</h2>
</section>

{{ create_macro.create_star(page, form, active_persona, controlled_personas) }}

{% endblock %}
