{% extends "base.html" %}

{% block content %}
{% if persona %}
<section class="col1 row1 w6 h2">
    <h2>Add {{ persona.username }} as a contact</h2>
    <form method="POST" action="{{url_for('contact_request')}}">
        <input type="submit" value="Add contact" />
    </form>
</section>
{% else %}
<section class="col1 row1 w6 h2">
    <h2>Error: Unknown Persona.</h2>
</section>
{% endif %}
{% endblock %}