{% extends "base.html" %}
{% import "macros/star.html" as star_macros %}
{% import "macros/planet.html" as planet_macros %}

{% block content %}

{% for star in page.stars %}
{% for planet in star.content.planets %}
<section class='{{ star.css_class }}'>
    {{ planet_macros.planet(planet, False)}}
</section>
{% endfor %}

<section class="{%if star.content.planets|length() >= 1 %}image-overlay{% endif %} {{ star.css_class }}">
    {{ star_macros.star(star.content) }}
</section>

{% endfor %}

{% endblock %}
