{% extends "base.html" %}

{% block content %}
<section class="col1 row1 w4 h1">
<h2><i class="fa fa-plus"></i> Create Persona</h2>
</section>

<!-- <section class="col1 row2 w4 h6">
<form method="POST" action="{{url_for('create_persona')}}">
    {{ form.name.label}} {{ form.name(size=20) }} <br />
    {% if form.name.errors %}
        <ul class="errors">{% for error in form.name.errors %}<li>{{ error }}</li>{% endfor %}</ul>
    {% endif %}

    {{ form.email.label}} {{ form.email(size=20) }}<br />
    {% if form.email.errors %}
        <ul class="errors">{% for error in form.email.errors %}<li>{{ error }}</li>{% endfor %}</ul>
    {% endif %}
    {{ form.csrf_token }}
    <input type="submit" value="Create">
</form>
</section> -->


<section class="col1 row2 w4 h6">
    <form method="POST" action="{{url_for('create_persona')}}">
        <div class="control-group {% if form.name.errors %}error{% endif %}">
            <div class="controls input-prepend">
                <span class="add-on"><i class="fa fa-circle"></i></span>
                {{ form.name(placeholder="Username")}}
            </div>
            {% if form.name.errors %}
                <ul class="errors">{% for error in form.name.errors %}<li>{{ error }}</li>{% endfor %}</ul>
            {% endif %}
        </div>
        <div class="control-group {% if form.email.errors %}error{% endif %}">
            <div class="controls input-prepend">
                <span class="add-on"><i class="fa fa-envelope"></i></span>
                {{ form.email(placeholder="Email")}}
            </div>
            {% if form.email.errors %}
                <ul class="errors">{% for error in form.email.errors %}<li>{{ error }}</li>{% endfor %}</ul>
            {% endif %}
        </div>
        <div class="control-group">
            {{ form.csrf_token }}
            <div class="controls">
                <button type="submit" class="btn">Create</button>
            </div>
        </div>
    </form>
</section>
{% endblock %}